package main

import (
	"database/sql"
	"os"
    "encoding/json"
    "strconv"
    migration "bitbucket.org/prmunty/util/dbmigration/db"
	_ "github.com/mattn/go-sqlite3"
    sqlite "bitbucket.org/prmunty/util/dbmigration/sqliteutils"

)

func main() {
	const version float64 = 5.0
	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		return migration.DoUntilErr(tx,
			UpdateAnalyticsConfiguration,
			MigrateConfigSettings,
		)
	})
}

var (
	AppConfig *AppConfiguration
)

type AppConfiguration struct {

	App struct {
		Environment string
		DataMode int
		Transmit struct {
			Frequency int
		}
		Retry struct {
			Attempts  int
			Frequency int
		}
		Timeout int
		Ntp     struct {
			Server string
			Sync   struct {
				Frequency int
			}
		}
	}
	Azure struct {
		Hubname string
		Port    int
		Device  struct {
			Id    string
			Topic string
		}
	}
	Network struct {
		Proxy struct {
			Enabled        bool
			Address        string
			Authentication struct {
				Enabled bool
				Type    string
				Basic   struct {
					Username string
					Password string
				}
				Bearer struct {
					Token string
				}
			}
			Security struct {
				Enabled     bool
				Type        string
				RootCaPath  string // will need to think about this
				Certificate struct {
					Path    string
					KeyPath string
				}
			}
		}
	}
	Data struct {
		Analog struct {
			Enabled   bool
			Frequency int
		}
		Digital struct {
			Enabled   bool
			Frequency int
		}
		Calculated struct {
			Enabled   bool
			Frequency int
		}
		Mode struct {
			Enabled   bool
			Frequency int
		}
		Setpoint struct {
			Enabled   bool
			Frequency int
		}
		Timer struct {
			Enabled   bool
			Frequency int
		}
		Process struct {
			Enabled   bool
			Frequency int
		}
		Alarms struct {
			Enabled   bool
			Frequency int
		}
	}
}

func InitConfig() error {
	config, err := ParseConfig("/uniq/comms/comms.config")
	AppConfig = config

	return err

}


func MigrateConfigSettings(tx *sql.Tx) error {
    //attempt to read the config file
	err := InitConfig()
	//if the config file exists and loads successfully, migrate settings
	if err == nil {
		proxy_enable:=strconv.FormatBool(AppConfig.Network.Proxy.Enabled)
		_, err = tx.Exec("UPDATE analytics_configuration SET config_value='" + proxy_enable +"' WHERE config_key='proxy_enable';")
		if err != nil {
			return err
		}
		_, err = tx.Exec("UPDATE analytics_configuration SET config_value='" + AppConfig.Network.Proxy.Address +"' WHERE config_key='proxy_address';")
		if err != nil {
			return err
		}
		proxy_auth_enable:=strconv.FormatBool(AppConfig.Network.Proxy.Authentication.Enabled)
		_, err = tx.Exec("UPDATE analytics_configuration SET config_value='" + proxy_auth_enable +"' WHERE config_key='proxy_authentication_enabled';")
		if err != nil {
			return err
		}
		_, err = tx.Exec("UPDATE analytics_configuration SET config_value='" + AppConfig.Network.Proxy.Authentication.Type +"' WHERE config_key='proxy_authentication_type';")
		if err != nil {
			return err
		}
		_, err = tx.Exec("UPDATE analytics_configuration SET config_value='" + AppConfig.Network.Proxy.Authentication.Basic.Username +"' WHERE config_key='proxy_authentication_username';")
		if err != nil {
			return err
		}
		_, err = tx.Exec("UPDATE analytics_configuration SET config_value='" + AppConfig.Network.Proxy.Authentication.Basic.Password +"' WHERE config_key='proxy_authentication_password';")
		if err != nil {
			return err
		}
		_, err = tx.Exec("UPDATE analytics_configuration SET config_value='" + AppConfig.Network.Proxy.Authentication.Bearer.Token +"' WHERE config_key='proxy_authentication_bearer';")
		if err != nil {
			return err
		}
	}else{

	    keys := []string{"environment","global_transmit_frequency","global_retry_limit","global_retry_limit_frequency",
        		"timeout","time_sync_method","ntp_server","ntp_sync","http_time_server", "http_sync","last_sync","azure_hubname","azure_hubname_port","device_id","device_topic",
        		"proxy_enable","proxy_address","proxy_authentication_enabled","proxy_authentication_type","proxy_authentication_username","proxy_authentication_password",
        		"proxy_authentication_bearer","analog_enabled","analog_frequency","digital_enabled","digital_frequency", "calculated_enabled","calculated_frequency", "mode_enabled",
        		"mode_frequency","setpoint_enabled","setpoint_frequency","timer_enabled","timer_frequency","process_enabled","process_frequency","alarms_enabled","alarms_frequency"}

        values := []string{"Production","5000","3","10000","1800000","0","1.us.pool.ntp.org","300","localhost","300","","aih-devicehub-usc-prod-b0ae.azure-devices.net",
        		"8883", "","","false","","false","","","","","true","1000","true","1000","true","1000","false","1000","true","60000","false","1000","false","1000","false","1000"}


        _, err = tx.Exec(`INSERT INTO analytics_configuration(config_key, config_value) VALUES( ?, ? )`, "Migration", "Success");
        if err != nil {
             _, _ = tx.Exec(`DROP TABLE analytics_configuration IF EXISTS`)
             _, err = tx.Exec(`CREATE TABLE analytics_configuration(config_key VARCHAR(25),config_value VARCHAR(50));`)
             for i, key := range keys {
                if _, err := tx.Exec(`INSERT INTO analytics_configuration(config_key, config_value) VALUES( ?, ? )`, key,values[i],
                    ); err != nil {
                        return err
                    }
                }
        }



	}
    //delete file and disregard error if it does not exist
    os.Remove("/uniq/comms/comms.config")

	return nil

}

func UpdateAnalyticsConfiguration(tx *sql.Tx) error {

	var err error
    //if the table does not exist, create it
	if !(sqlite.DB{tx}.TableExists("analytics_configuration")) {
		_, err = tx.Exec(`CREATE TABLE analytics_configuration(config_key VARCHAR(25),config_value VARCHAR(50));`)
	}

	if err != nil {
		return err
	}

	keys := []string{"environment","global_transmit_frequency","global_retry_limit","global_retry_limit_frequency",
		"timeout","time_sync_method","ntp_server","ntp_sync","http_time_server", "http_sync","last_sync","azure_hubname","azure_hubname_port","device_id","device_topic",
		"proxy_enable","proxy_address","proxy_authentication_enabled","proxy_authentication_type","proxy_authentication_username","proxy_authentication_password",
		"proxy_authentication_bearer","analog_enabled","analog_frequency","digital_enabled","digital_frequency", "calculated_enabled","calculated_frequency", "mode_enabled",
		"mode_frequency","setpoint_enabled","setpoint_frequency","timer_enabled","timer_frequency","process_enabled","process_frequency","alarms_enabled","alarms_frequency"}

	values := []string{"Production","5000","3","10000","1800000","0","1.us.pool.ntp.org","300","localhost","300","","aih-devicehub-usc-prod-b0ae.azure-devices.net",
		"8883", "","","false","","false","","","","","true","1000","true","1000","true","1000","false","1000","true","60000","false","1000","false","1000","false","1000"}

	for i, key := range keys {
		if _, err := tx.Exec(
			`INSERT OR IGNORE INTO analytics_configuration(config_key, config_value) VALUES( ?, ? )`, key,values[i],
		); err != nil {
			return err
		}
	}

	return nil
}


func ParseConfig(path string) (*AppConfiguration, error) {
	config := new(AppConfiguration)

	file, err := os.Open(path)
	if err != nil {
		return config, err
	}

	decoder := json.NewDecoder(file)
	err = decoder.Decode(&config)

	if err != nil {
		file.Close()
		return config, err
	}

	file.Close()

	return config, nil
}